==> snf_3.3.0_webrelease.tar.gz: snf_3.3.0_webrelease/src/snf/snf_dipol.f90 <==

!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_dipol
  use f90_kind
  use snf_error

  private
  public :: create_dipol_dxyz,print_dipol_dxyz,expand_dipol
  public :: destroy_dipol_dxyz, create_dipol_dq, symm_dipol
  public :: create_dipol_dq_intonly


  !************************************************************************

contains

  !************************************************************************

  subroutine create_dipol_dxyz(dipol_dxyz,dipol,distortion)
    implicit none
    !----------------------------------------------------------------
    ! calculation off first derivative of dipolmoment with 
    ! following formula:
    ! 
    !  d dipol    dipol(xi+e(xi) - dipol(xi-e(xi))  
    ! -------- = ---------------------------------   
    !    dxi                 2*e(xi) 
    !           
    ! unit is e since dipolmoment is e*bohr and distortion is also 
    ! given in bohr.
    !
    ! written by Carsten Kind and Markus Reiher, Uni Erlangen, 1999
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    ! (correction of 5-point central differences formula)
    ! extended by J. Neugebauer, Uni Erlangen, 2002 
    ! (introduction of 7-point central differences formula)
    !----------------------------------------------------------------
    real(double),pointer :: dipol_dxyz(:,:)
    real(double),intent(in) :: dipol(:,:,:)
    real(double),intent(in) :: distortion(:)
    integer :: idum
    allocate(dipol_dxyz(ubound(dipol,1),ubound(dipol,2)))
    idum=ubound(dipol,3)
    if (idum.eq.2) then
       dipol_dxyz(1,:) = 0.5d0/distortion(:)*(dipol(1,:,1)-dipol(1,:,2))
       dipol_dxyz(2,:) = 0.5d0/distortion(:)*(dipol(2,:,1)-dipol(2,:,2))
       dipol_dxyz(3,:) = 0.5d0/distortion(:)*(dipol(3,:,1)-dipol(3,:,2))
    else if (idum.eq.4) then   
       dipol_dxyz(1,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*dipol(1,:,1)&
        & +16.0d0*dipol(1,:,2)-16.0d0*dipol(1,:,3)+2.0d0*dipol(1,:,4))
       dipol_dxyz(2,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*dipol(2,:,1)&
        & +16.0d0*dipol(2,:,2)-16.0d0*dipol(2,:,3)+2.0d0*dipol(2,:,4))
       dipol_dxyz(3,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*dipol(3,:,1)&
        & +16.0d0*dipol(3,:,2)-16.0d0*dipol(3,:,3)+2.0d0*dipol(3,:,4))
    else if (idum.eq.6) then   
       dipol_dxyz(1,:) = 1.0d0/(720.0d0*distortion(:))*(-12.0d0*dipol(1,:,6)&
       & +108.0d0*dipol(1,:,5)-540.0d0*dipol(1,:,4)+540.0d0*dipol(1,:,3) &
       & -108.0d0*dipol(1,:,2)+12.0d0*dipol(1,:,1))
       dipol_dxyz(2,:) = 1.0d0/(720.0d0*distortion(:))*(-12.0d0*dipol(2,:,6)&
       & +108.0d0*dipol(2,:,5)-540.0d0*dipol(2,:,4)+540.0d0*dipol(2,:,3) &
       & -108.0d0*dipol(2,:,2)+12.0d0*dipol(2,:,1))
       dipol_dxyz(3,:) = 1.0d0/(720.0d0*distortion(:))*(-12.0d0*dipol(3,:,6)&
       & +108.0d0*dipol(3,:,5)-540.0d0*dipol(3,:,4)+540.0d0*dipol(3,:,3) &
       & -108.0d0*dipol(3,:,2)+12.0d0*dipol(3,:,1))
    else
       call serror('create_dipol_dxyz',2,'Wrong ctrl%numderiv value!')
    end if
  end subroutine create_dipol_dxyz

  !************************************************************************

  subroutine expand_dipol(dipol_dxyz,representation,symm_index,char_table)
    use snf_char_table, only : type_char_table
    use snf_symmat,only : symel_mat
    !----------------------------------------------------------------
    ! copy dipol moment derivatives calculated on irreducible atoms to the 
    ! reducible atoms. this is done by multiplying symmetry representations 
    ! in the dipol basis from the left and symmetry representations in the 
    ! derivate basis from the right with the dipol moment derivates 
    ! and selecting only the required columns.
    !
    ! written by Carsten Kind and Markus Reiher, Uni Erlangen, 1999
    !----------------------------------------------------------------
    implicit none
    type(type_char_table), intent(in) :: char_table
    real(double),pointer :: dipol_dxyz(:,:),rot_dipol(:,:),expanded_dipol(:,:)
    real(double),intent(in) :: representation(:,:,:)
    integer, intent(in) :: symm_index(:)
    integer :: s,i,offset,nvar,natoms
    nvar=ubound(dipol_dxyz,2)
    natoms=nvar/3
    allocate(rot_dipol(3,nvar),expanded_dipol(3,nvar))
    do s=1,char_table%nsymel
       rot_dipol=matmul(symel_mat(char_table%symel(s)),&
            &    matmul(dipol_dxyz,representation(:,:,s)))
       do i=1,natoms
          if(symm_index(i) .eq. s) then
             offset=(i-1)*3
             expanded_dipol(:,offset+1:offset+3)=rot_dipol(:,offset+1:offset+3)
          endif
       enddo
    enddo
    deallocate(dipol_dxyz,rot_dipol)
    dipol_dxyz => expanded_dipol
  end subroutine expand_dipol

  !************************************************************************

  subroutine symm_dipol(d_dxyz,rep,char_table)
    use snf_char_table, only : type_char_table
    use snf_symmat,only : symel_mat
    !----------------------------------------------------------------
    ! this subroutine transforms the dipole moment derivatives
    ! according to all symmetry operations of the point group.
    ! all resulting matrices are added and then diveded by the
    ! number of symmetrie operations. a symmetry check is performed.
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    type(type_char_table), intent(in) :: char_table
    real(double),pointer :: d_dxyz(:,:),rot_dipol(:,:),tmp_dip(:,:)
    real(double),intent(in) :: rep(:,:,:)
    real(double) :: errsum, fac
    character(len=60) :: errbuff 
    integer :: s,i,nvar,j
    nvar=ubound(d_dxyz,2)
    allocate(rot_dipol(3,nvar),tmp_dip(3,nvar))
    tmp_dip=0.0d0
    do s=1,char_table%nsymel
       rot_dipol=matmul(symel_mat(char_table%symel(s)),&
            &    matmul(d_dxyz,rep(:,:,s)))
       tmp_dip=tmp_dip+rot_dipol
       errsum=0.0d0
       do i=1,3
         do j=1,nvar
           errsum=errsum+abs(rot_dipol(i,j)-d_dxyz(i,j))
         end do
       enddo
       errsum=errsum/dble(nvar)/3.0d0
       if(errsum.gt.1.0d-3) then
         write(errbuff,'(a,f8.4,a,i3)') 'average error = ',errsum, ' for s = ',s
         call serror('symm_dipol',0,trim(adjustl(errbuff)))
       end if
    enddo
    fac=1.0d0/dble(char_table%nsymel)
    d_dxyz=fac*tmp_dip
    deallocate(rot_dipol,tmp_dip)
  end subroutine symm_dipol

  !************************************************************************

  subroutine print_dipol_dxyz(dipol_dxyz,filename)
    implicit none
    !----------------------------------------------------------------
    ! output routine
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    character*(*) :: filename
    integer :: i,k
    real(double),intent(in) :: dipol_dxyz(:,:)

    open(unit=45,file=filename,status='unknown')
    write(45,*) 'cartesian dipole moment derivatives'
    write(45,'(3d18.10)') ((dipol_dxyz(i,k),i=1,ubound(dipol_dxyz,1)),k=1,ubound(dipol_dxyz,2))
    close (45)
  end subroutine print_dipol_dxyz

  !************************************************************************

  subroutine destroy_dipol_dxyz(dipol_dxyz)
    !----------------------------------------------------------------
    ! clean up memory
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    implicit none
    real(double),pointer ::  dipol_dxyz(:,:)
    if(associated(dipol_dxyz)) deallocate(dipol_dxyz)
  end subroutine destroy_dipol_dxyz

  !************************************************************************

  subroutine create_dipol_dq(dipol_dxyz,norm_modes,molecule,dipol_dq)
    !----------------------------------------------------------------
    ! Transform dipole derivative to cartesian normal coordinates
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    use snf_molecule, only: type_atom
    implicit none
    real(double),intent(in) :: dipol_dxyz(:,:),norm_modes(:,:)
    type(type_atom), pointer :: molecule(:)
    real(double),pointer :: dipol_dq(:,:)
    integer :: i,j,k,nvar,irrep_dim

    nvar=ubound(norm_modes,1)
    irrep_dim=ubound(norm_modes,2)
    allocate(dipol_dq(3,irrep_dim))
    dipol_dq=0.d0
    do k=1,3
       do i=1,nvar
          do j=1,irrep_dim
             dipol_dq(k,j)= dipol_dq(k,j) + dipol_dxyz(k,i)*norm_modes(i,j)/ &
                  &                         dsqrt(molecule((i+2)/3)%mass) 
          enddo
       enddo
    enddo
  end subroutine create_dipol_dq

  !************************************************************************

  subroutine create_dipol_dq_intonly(dipol_dqi,dipol,nummodes,normmodes,cstep,numderiv,natoms,molecule)
    use snf_molecule
    implicit none
    !----------------------------------------------------------------
    ! calculation off first derivative of dipolmoment with
    ! respect to non mass-weighted normal coordinates using the 
    ! following formula:
    !
    !  d dipol    dipol(qi+e(qi) - dipol(qi-e(qi))
    ! -------- = ---------------------------------
    !    dqi                 2*distortion
    !
    ! unit is e since dipolmoment is e*bohr and distortion is also
    ! given in bohr.
    !
    ! written by C. Herrmann, Uni Bonn 2005, 
    ! based on create_dipol_dxyz, written by Carsten Kind and
    ! Markus Reiher, J. Neugebauer, Uni Erlangen, 1999-2002
    !----------------------------------------------------------------
    real(double),pointer :: dipol_dqi(:,:)
    real(double),pointer :: dipol(:,:,:)
    real(double) :: cstep
    integer :: i,j,natoms,nummodes,numderiv
    real(double) :: normmodes(natoms*3,natoms*3)
    real(double) :: zthres, sum, red_mass(nummodes)
    type(type_atom),pointer :: molecule(:)

    red_mass(:) = 0.d0
    
    do i = 1,3
    if (numderiv.eq.1) then
       dipol_dqi(i,:) = 0.5d0/cstep*(dipol(i,:,1)-dipol(i,:,2))
    else if (numderiv.eq.2) then
       dipol_dqi(i,:) = 1.0d0/(24.0d0*cstep)*(-2.0d0*dipol(i,:,1)&
        & +16.0d0*dipol(i,:,2)-16.0d0*dipol(i,:,3)+2.0d0*dipol(i,:,4))
    else if (numderiv.eq.3) then
       dipol_dqi(i,:) = 1.0d0/(720.0d0*cstep)*(-12.0d0*dipol(i,:,6)&
       & +108.0d0*dipol(i,:,5)-540.0d0*dipol(i,:,4)+540.0d0*dipol(i,:,3) &
       & -108.0d0*dipol(i,:,2)+12.0d0*dipol(i,:,1))
    else
       call serror('create_dipol_dq_intonly',2,'Wrong ctrl%numderiv value!')
    end if
    end do
      
! calculate reduced mass
    do i = 1,nummodes
      do j=1,ubound(molecule,1)*3
          red_mass(i) = red_mass(i) + &
               & (normmodes(i,j)**2 &
               & /molecule((j+2)/3)%mass)
       enddo
       red_mass(i)=1.d0/red_mass(i)

! divide derivative by reduced mass
       dipol_dqi(:,i) = dipol_dqi(:,i)/dsqrt(red_mass(i))
     end do

  end subroutine create_dipol_dq_intonly

end module snf_dipol

==> snf-4.0.0_webrelease.tar.gz: snf-4.0.0_webrelease/src/snf/snf_dipol.f90 <==

!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_dipol
  use f90_kind
  use snf_error

  private
  public :: create_dipol_dxyz,print_dipol_dxyz,expand_dipol
  public :: destroy_dipol_dxyz, create_dipol_dq, symm_dipol
  public :: create_dipol_dq_intonly


  !************************************************************************

contains

  !************************************************************************

  subroutine create_dipol_dxyz(dipol_dxyz,dipol,distortion)
    implicit none
    !----------------------------------------------------------------
    ! calculation off first derivative of dipolmoment with 
    ! following formula:
    ! 
    !  d dipol    dipol(xi+e(xi) - dipol(xi-e(xi))  
    ! -------- = ---------------------------------   
    !    dxi                 2*e(xi) 
    !           
    ! unit is e since dipolmoment is e*bohr and distortion is also 
    ! given in bohr.
    !
    ! written by Carsten Kind and Markus Reiher, Uni Erlangen, 1999
    ! modified by J. Neugebauer, Uni Erlangen, 2001
    ! (correction of 5-point central differences formula)
    ! extended by J. Neugebauer, Uni Erlangen, 2002 
    ! (introduction of 7-point central differences formula)
    !----------------------------------------------------------------
    real(double),pointer :: dipol_dxyz(:,:)
    real(double),intent(in) :: dipol(:,:,:)
    real(double),intent(in) :: distortion(:)
    integer :: idum
    allocate(dipol_dxyz(ubound(dipol,1),ubound(dipol,2)))
    idum=ubound(dipol,3)
    if (idum.eq.2) then
       dipol_dxyz(1,:) = 0.5d0/distortion(:)*(dipol(1,:,1)-dipol(1,:,2))
       dipol_dxyz(2,:) = 0.5d0/distortion(:)*(dipol(2,:,1)-dipol(2,:,2))
       dipol_dxyz(3,:) = 0.5d0/distortion(:)*(dipol(3,:,1)-dipol(3,:,2))
    else if (idum.eq.4) then   
       dipol_dxyz(1,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*dipol(1,:,1)&
        & +16.0d0*dipol(1,:,2)-16.0d0*dipol(1,:,3)+2.0d0*dipol(1,:,4))
       dipol_dxyz(2,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*dipol(2,:,1)&
        & +16.0d0*dipol(2,:,2)-16.0d0*dipol(2,:,3)+2.0d0*dipol(2,:,4))
       dipol_dxyz(3,:) = 1.0d0/(24.0d0*distortion(:))*(-2.0d0*dipol(3,:,1)&
        & +16.0d0*dipol(3,:,2)-16.0d0*dipol(3,:,3)+2.0d0*dipol(3,:,4))
    else if (idum.eq.6) then   
       dipol_dxyz(1,:) = 1.0d0/(720.0d0*distortion(:))*(-12.0d0*dipol(1,:,6)&
       & +108.0d0*dipol(1,:,5)-540.0d0*dipol(1,:,4)+540.0d0*dipol(1,:,3) &
       & -108.0d0*dipol(1,:,2)+12.0d0*dipol(1,:,1))
       dipol_dxyz(2,:) = 1.0d0/(720.0d0*distortion(:))*(-12.0d0*dipol(2,:,6)&
       & +108.0d0*dipol(2,:,5)-540.0d0*dipol(2,:,4)+540.0d0*dipol(2,:,3) &
       & -108.0d0*dipol(2,:,2)+12.0d0*dipol(2,:,1))
       dipol_dxyz(3,:) = 1.0d0/(720.0d0*distortion(:))*(-12.0d0*dipol(3,:,6)&
       & +108.0d0*dipol(3,:,5)-540.0d0*dipol(3,:,4)+540.0d0*dipol(3,:,3) &
       & -108.0d0*dipol(3,:,2)+12.0d0*dipol(3,:,1))
    else
       call serror('create_dipol_dxyz',2,'Wrong ctrl%numderiv value!')
    end if
  end subroutine create_dipol_dxyz

  !************************************************************************

  subroutine expand_dipol(dipol_dxyz,representation,symm_index,char_table)
    use snf_char_table, only : type_char_table
    use snf_symmat,only : symel_mat
    !----------------------------------------------------------------
    ! copy dipol moment derivatives calculated on irreducible atoms to the 
    ! reducible atoms. this is done by multiplying symmetry representations 
    ! in the dipol basis from the left and symmetry representations in the 
    ! derivate basis from the right with the dipol moment derivates 
    ! and selecting only the required columns.
    !
    ! written by Carsten Kind and Markus Reiher, Uni Erlangen, 1999
    !----------------------------------------------------------------
    implicit none
    type(type_char_table), intent(in) :: char_table
    real(double),pointer :: dipol_dxyz(:,:),rot_dipol(:,:),expanded_dipol(:,:)
    real(double),intent(in) :: representation(:,:,:)
    integer, intent(in) :: symm_index(:)
    integer :: s,i,offset,nvar,natoms
    nvar=ubound(dipol_dxyz,2)
    natoms=nvar/3
    allocate(rot_dipol(3,nvar),expanded_dipol(3,nvar))
    do s=1,char_table%nsymel
       rot_dipol=matmul(symel_mat(char_table%symel(s)),&
            &    matmul(dipol_dxyz,representation(:,:,s)))
       do i=1,natoms
          if(symm_index(i) .eq. s) then
             offset=(i-1)*3
             expanded_dipol(:,offset+1:offset+3)=rot_dipol(:,offset+1:offset+3)
          endif
       enddo
    enddo
    deallocate(dipol_dxyz,rot_dipol)
    dipol_dxyz => expanded_dipol
  end subroutine expand_dipol

  !************************************************************************

  subroutine symm_dipol(d_dxyz,rep,char_table)
    use snf_char_table, only : type_char_table
    use snf_symmat,only : symel_mat
    !----------------------------------------------------------------
    ! this subroutine transforms the dipole moment derivatives
    ! according to all symmetry operations of the point group.
    ! all resulting matrices are added and then diveded by the
    ! number of symmetrie operations. a symmetry check is performed.
    !
    ! written by Johannes Neugebauer, Uni Erlangen, 2001
    !----------------------------------------------------------------
    implicit none
    type(type_char_table), intent(in) :: char_table
    real(double),pointer :: d_dxyz(:,:),rot_dipol(:,:),tmp_dip(:,:)
    real(double),intent(in) :: rep(:,:,:)
    real(double) :: errsum, fac
    character(len=60) :: errbuff 
    integer :: s,i,nvar,j
    nvar=ubound(d_dxyz,2)
    allocate(rot_dipol(3,nvar),tmp_dip(3,nvar))
    tmp_dip=0.0d0
    do s=1,char_table%nsymel
       rot_dipol=matmul(symel_mat(char_table%symel(s)),&
            &    matmul(d_dxyz,rep(:,:,s)))
       tmp_dip=tmp_dip+rot_dipol
       errsum=0.0d0
       do i=1,3
         do j=1,nvar
           errsum=errsum+abs(rot_dipol(i,j)-d_dxyz(i,j))
         end do
       enddo
       errsum=errsum/dble(nvar)/3.0d0
       if(errsum.gt.1.0d-3) then
         write(errbuff,'(a,f8.4,a,i3)') 'average error = ',errsum, ' for s = ',s
         call serror('symm_dipol',0,trim(adjustl(errbuff)))
       end if
    enddo
    fac=1.0d0/dble(char_table%nsymel)
    d_dxyz=fac*tmp_dip
    deallocate(rot_dipol,tmp_dip)
  end subroutine symm_dipol

  !************************************************************************

  subroutine print_dipol_dxyz(dipol_dxyz,filename)
    implicit none
    !----------------------------------------------------------------
    ! output routine
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    character*(*) :: filename
    integer :: i,k
    real(double),intent(in) :: dipol_dxyz(:,:)

    open(unit=45,file=filename,status='unknown')
    write(45,*) 'cartesian dipole moment derivatives'
    write(45,'(3d18.10)') ((dipol_dxyz(i,k),i=1,ubound(dipol_dxyz,1)),k=1,ubound(dipol_dxyz,2))
    close (45)
  end subroutine print_dipol_dxyz

  !************************************************************************

  subroutine destroy_dipol_dxyz(dipol_dxyz)
    !----------------------------------------------------------------
    ! clean up memory
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    implicit none
    real(double),pointer ::  dipol_dxyz(:,:)
    if(associated(dipol_dxyz)) deallocate(dipol_dxyz)
  end subroutine destroy_dipol_dxyz

  !************************************************************************

  subroutine create_dipol_dq(dipol_dxyz,norm_modes,molecule,dipol_dq)
    !----------------------------------------------------------------
    ! Transform dipole derivative to cartesian normal coordinates
    !
    ! written by Carsten Kind, Uni Erlangen, 1999
    !----------------------------------------------------------------
    use snf_molecule, only: type_atom
    implicit none
    real(double),intent(in) :: dipol_dxyz(:,:),norm_modes(:,:)
    type(type_atom), pointer :: molecule(:)
    real(double),pointer :: dipol_dq(:,:)
    integer :: i,j,k,nvar,irrep_dim

    nvar=ubound(norm_modes,1)
    irrep_dim=ubound(norm_modes,2)
    allocate(dipol_dq(3,irrep_dim))
    dipol_dq=0.d0
    do k=1,3
       do i=1,nvar
          do j=1,irrep_dim
             dipol_dq(k,j)= dipol_dq(k,j) + dipol_dxyz(k,i)*norm_modes(i,j)/ &
                  &                         dsqrt(molecule((i+2)/3)%mass) 
          enddo
       enddo
    enddo
  end subroutine create_dipol_dq

  !************************************************************************

  subroutine create_dipol_dq_intonly(dipol_dqi,dipol,nummodes,normmodes,cstep,numderiv,natoms,molecule)
    use snf_molecule
    implicit none
    !----------------------------------------------------------------
    ! calculation of first derivative of dipolmoment with
    ! respect to mass-weighted normal coordinates using the 
    ! following formula:
    !
    !  d dipol    dipol(qi+e(qi) - dipol(qi-e(qi))
    ! -------- = ---------------------------------
    !    dqi                 2*distortion
    !
    ! unit is e since dipolmoment is e*bohr and distortion is also
    ! given in bohr.
    !
    ! written by C. Herrmann, Uni Bonn 2005, 
    ! based on create_dipol_dxyz, written by Carsten Kind and
    ! Markus Reiher, J. Neugebauer, Uni Erlangen, 1999-2002
    !----------------------------------------------------------------
    real(double),pointer :: dipol_dqi(:,:)
    real(double),pointer :: dipol(:,:,:)
    real(double) :: cstep
    integer :: i,j,natoms,nummodes,numderiv
    real(double) :: normmodes(natoms*3,natoms*3)
    real(double) :: zthres, sum, red_mass(nummodes)
    type(type_atom),pointer :: molecule(:)

    red_mass(:) = 0.d0
    
    do i = 1,3
    if (numderiv.eq.1) then
       dipol_dqi(i,:) = 0.5d0/cstep*(dipol(i,:,1)-dipol(i,:,2))
    else if (numderiv.eq.2) then
       dipol_dqi(i,:) = 1.0d0/(24.0d0*cstep)*(-2.0d0*dipol(i,:,1)&
        & +16.0d0*dipol(i,:,2)-16.0d0*dipol(i,:,3)+2.0d0*dipol(i,:,4))
    else if (numderiv.eq.3) then
       dipol_dqi(i,:) = 1.0d0/(720.0d0*cstep)*(-12.0d0*dipol(i,:,6)&
       & +108.0d0*dipol(i,:,5)-540.0d0*dipol(i,:,4)+540.0d0*dipol(i,:,3) &
       & -108.0d0*dipol(i,:,2)+12.0d0*dipol(i,:,1))
    else
       call serror('create_dipol_dq_intonly',2,'Wrong ctrl%numderiv value!')
    end if
    end do
      
! calculate reduced mass
    do i = 1,nummodes
      do j=1,ubound(molecule,1)*3
          red_mass(i) = red_mass(i) + &
               & (normmodes(i,j)**2 &
               & /molecule((j+2)/3)%mass)
       enddo
       red_mass(i)=1.d0/red_mass(i)

! divide derivative by reduced mass
       dipol_dqi(:,i) = dipol_dqi(:,i)/dsqrt(red_mass(i))
     end do

  end subroutine create_dipol_dq_intonly

end module snf_dipol
